#include <iostream>
#include <string>

#if 1
#include "GenUtils/smart_ptr.h"
#else
#include <boost/shared_ptr.hpp>
#define smart_ptr boost::shared_ptr
struct smart_base {};
#endif

#include <boost/python.hpp>
using namespace boost::python;

/*****************************************************************************
*   Test classes
*****************************************************************************/
class Base: public smart_base
{
public:
    virtual std::string getName() const { return "Base"; }
};

class SubClass: public Base
{
public:
    virtual std::string getName() const { return "SubClass"; }
};

class Interface
{
public:
    void method(smart_ptr<Base> arg) const
    {
        std::cout << "Interface::method(" << arg->getName() << ")\n";
    }
};

/*****************************************************************************
*   The extension module
*****************************************************************************/
BOOST_PYTHON_MODULE(Extension)
{
    Interface worker;
    smart_ptr<SubClass> objp = new SubClass();
    worker.method(objp);

    class_<Interface>("Interface")
        // methods
        .def("method", &Interface::method)
    ;

    class_<Base, smart_ptr<Base>, boost::noncopyable>("Base", no_init)
        .def("getName", &Base::getName)
    ;

    class_<SubClass, bases<Base> >("SubClass")
        .def("getName", &SubClass::getName)
    ;
}
